/* NoSuch MIDI - Copyright 1997 by Tim Thompson */

package com.nosuch.midi;

/**
 * A MidiNote is a MidiMsg that has a duration, and consists of a note-on
 * and a note-off.  By packaging the two MIDI messages in a single item,
 * algorithmic stuff is easier.  The MidiSystem.schedule() method automatically
 * handles the scheduling of both pieces of a MidiNote message.
 *
 * @author	Tim Thompson
 */

public class MidiNote extends MidiMsg implements Cloneable, MidiNoteable {

	private int pitch;
	private int channel;
	private int velocity;
	private long duration;

	static private long defDuration = 1;
	static private int defVelocity = 64;
	static private int defChannel = 1;

	/** Constructor
	 * @param t time (in clicks)
	 * @param p pitch (0-127)
	 * @param c channel (1-16)
	 * @param v velocity (0-127)
	 * @param d duration (in clicks)
	 */
	public MidiNote(int p, long t, int c, int v, long d) {
		super(t);
		pitch = p;
		channel = c;
		if ( c <= 0 || c > 16 )
			System.err.println("Warning, Midinote created with channel value ("+Integer.toString(c)+") that isn't 1-16");
		velocity = v;
		duration = d;
	}

	/** Constructor
	 * @param p pitch (0-127)
	 * The default values for channel, velocity, and duration are used.
	 * The default values can be changed with defaultChannel(),
	 * defaultVelocity(), and defaultDuration().
	 */
	public MidiNote(int p) {
		this(p,(long)0,defChannel,defVelocity,defDuration);
	}

	/**
	 * Return a clone of a MidiNote.  Note that "next" is *not* cloned.
	 */
	public Object clone() {
		return new MidiNote(pitch,time,channel,velocity,duration);
	}

	public boolean equals(MidiMsg m) {
		if ( ! super.equals(m) )
			return false;
		if ( ! ( m instanceof MidiNote) )
			return false;
		MidiNote mn = (MidiNote)m;
		if ( mn.pitch != pitch )
			return false;
		if ( mn.channel != channel )
			return false;
		if ( mn.velocity != velocity )
			return false;
		if ( mn.duration != duration )
			return false;
		return true;
	}

	/** Set the default duration used when MidiNotes are created.
	 * @param d duration (in clicks)
	 */
	static public void defaultDuration(long d) {
		defDuration = d;
	}

	/** Set the default channel used when MidiNotes are created.
	 * @param c channel (1-16)
	 */
	static public void defaultChannel(int c) {
		defChannel = c;
	}

	/** Set the default velocity used when MidiNotes are created.
	 * @param v velocity (0-127)
	 */
	static public void defaultVelocity(int v) {
		defVelocity = v;
	}

	/** Get the pitch (0-127) */
	public int getPitch() { return pitch; }

	/** Set the pitch (0-127) */
	public void setPitch(int p) { pitch = p; }

	/** Set the duration (in clicks) */
	public void setDuration(long d) { duration = d; }

	/** Get the duration (in clicks) */
	public long getDuration() { return duration; }

	/** Get the channel (1-16) */
	public int getChannel() { return channel; }

	/** Set the channel (1-16) */
	public void setChannel(int c) {
		if ( c <= 0 || c > 16 )
			System.err.println("Warning, Midinote.setChannel("+Integer.toString(c)+") ignored - channel must be 1-16");
		else
			channel = c;
	}

	/** Get the velocity (0-127) */
	public int getVelocity() { return velocity; }

	/** Set the velocity (0-127) */
	public void setVelocity(int v) { velocity = v; }

	/** Get the bytes representing the start of the note */
	public byte[] getBytesOn() {
		byte[] b = new byte[3];
		b[0] = (byte)(0x90 + (channel-1));
		b[1] = (byte)pitch;
		b[2] = (byte)velocity;
		return b;
	}

	/** Get the bytes representing the ending of the note */
	public byte[] getBytesOff() {
		byte[] b = new byte[3];
		b[0] = (byte)(0x80 + (channel-1));
		b[1] = (byte)pitch;
		b[2] = (byte)0;
		return b;
	}

	/** Get a human-readable version of the note */
	public String toString() {
		String s = "MidiNote(t="+Long.toString(time)
			+",p="+Integer.toString(pitch)
			+",c="+Integer.toString(channel)
			+",v="+Integer.toString(velocity)
			+",d="+Long.toString(duration)
			+")";
		// if ( next != null )
		// 	s += "+";
		return s;
	}
}
